/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: IconRead.c,v 1.4 1993/10/25 20:47:29 dta Exp $
 **
 **  $Source: /cvs/lib/xn/IconRead.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <malloc.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <Xm/Xm.h>

#include "Xn.h"

#if defined(sco) || defined(InterActive)
#define SwapBytes(s) s
#else
#define SwapBytes(s) ((0x00ff & (s >> 8) | ((s << 8) & 0xff00)))
#endif

typedef struct {
    short            reserved;/* Reserved, must be set to 0	 */
    short            type;	/* Resource type, 1 for icons */
    short            count;	/* Number of images in the file */
}               IconHeader;

typedef struct {
    char            Width;	/* Width in pixels */
    char            Height;	/* Height in pixels */
    char            ColorCount;	/* Number of colors */
    char            Reserved1;
    short           Reserved2;
    short           Reserved3;
    short           Size;	/* Size in bytes of the pixel array */
    short           Reserved4;
    short           Offset; /* Offset in bytes from beginning of file */
    short           Reserved5;
}               IconFile;

Pixmap XnIconRead (widget, filename, width, height, bg, ratio)
Widget	widget;
char   *filename;
int    *width;
int    *height;
Pixel	bg;
double 	ratio;
{
    Display	       *display = XtDisplay(widget);
    XImage 	       *image;
    Pixmap 		pixmap;
    GC 			gc;
    IconHeader      	header;
    IconFile        	icon;
    struct stat 	st;
    unsigned char      *ptr;
    unsigned char      *mask, *data, *colors;
    Pixel	 	msk,   src,   dst;
    int			fd;
    int 		xx, yy;
    int 		x,  y;
    int 		i;
    XnColorRequest	color_reqs[16];
    Pixel		pallette[16];
    char pathname [256];
    
    if (getenv ("ICONDIR"))
	sprintf (pathname, "%s/%s", getenv ("ICONDIR"), filename);
    else
	strcpy (pathname, filename);

    if (stat (pathname, &st) < 0)
	return (0);

    if ((fd = open (pathname, O_RDONLY)) < 0) {
	fprintf (stderr, "Error opening input file %s\n", pathname);
	return (0);
    }

    if (!(ptr = (unsigned char *)malloc (st.st_size)))
	return (0);
    if (read (fd, ptr, st.st_size) < 0)
	return 0;
    close (fd);

    memcpy ((char *) &header, ptr, sizeof (IconHeader));
    memcpy ((char *) &icon, ptr + sizeof (IconHeader), sizeof (IconFile));

    if (header.reserved) {
#ifdef DEBUG_ICON_READ
	printf ("header.reserved = %d\n", header.reserved);
#endif
	return (0);
    }

    header.type  = SwapBytes (header.type);
    header.count = SwapBytes (header.count);
    icon.Size    = SwapBytes (icon.Size);
    icon.Offset  = SwapBytes (icon.Offset);

#ifdef DEBUG_ICON_READ
    if (st.st_size - icon.Offset - icon.Size) {
        printf ("sizes wrong %d %s\n", st.st_size - icon.Offset - icon.Size, pathname);
    }
#endif

    /*
     * Work from end of file since we can't seem to decode all of the icon header.
     * header
     * colors
     * data
     * mask
     */
    mask = ptr + st.st_size - icon.Width * icon.Height/8; /* each byte holds 8 points */
    data = mask - (icon.Width * icon.Height) / 2;	 /* each byte holds 2 points */
    colors = data - 16 * 4;				 /* each color is (b, g, r, reserved) */

    for (i = 0; i < 16; i++) {
	color_reqs[i].color_name = NULL;
	color_reqs[i].blue  = colors[i*4 + 0]*256;
	color_reqs[i].green = colors[i*4 + 1]*256;
	color_reqs[i].red   = colors[i*4 + 2]*256;
	color_reqs[i].addr  = &pallette[i];
	color_reqs[i].priority = 0;
    }
    XnRequestColors(widget, color_reqs, 16);
	
#ifdef DEBUG_ICON_READ
    i = 0;
    while (colors < data) {
	printf("%02x ", (unsigned char)*colors);
	if (i == 3) {
	    i = 0;
	    printf("\n");
	} else {
	    i++;
	}
	colors++;
    }

    printf ("type %d count %d\n",
	header.type,
	header.count);

    printf ("%d x %d colors %d\n",
	icon.Width,
	icon.Height,
	icon.ColorCount);

    printf ("reserved %d %d %d\n",
	icon.Reserved1,
	icon.Reserved2,
	icon.Reserved3);

    printf ("size %d offset %d bytes %d, %d, real %d\n",
	icon.Size,
	icon.Offset,
	icon.Width * icon.Height,
	st.st_size - icon.Offset - icon.Size,
	st.st_size - 128 - (icon.Width * icon.Height) / 2);

    printf ("ptr + off: %x, data %x\n", ptr  + icon.Offset, data);
    /*
     * Last 128 bytes are mask 32 x 32 x 1
     */
    mask = ptr + st.st_size - 128;
    for (y = 31; y >= 0; y--) {
	for (x = 0; x < 32; x++) {
	    if (mask[y*4 + x/8] & (0x80 >> (x % 8)))
		printf("1");
	    else
		printf("0");
	}
	printf("\n");
    }
#endif /* DEBUG_ICON_READ */

    *width = (int)(icon.Width * ratio);
    *height = (int)(icon.Height * ratio);

    gc = DefaultGC (display, DefaultScreen (display));

    image = XCreateImage (display, DefaultVisual (display, DefaultScreen (display)),
	DefaultDepth (display, DefaultScreen (display)), ZPixmap, 0, (char *) NULL,
	*width, *height,
	BitmapPad (display), 0);

    if (image == (XImage *) NULL)
	return (0);

    image->data = (char *) malloc (image->bytes_per_line * *height * 2);

    for (y = *height-1; y >= 0; y--) {
	yy = (int)(y / ratio);
	for (x = 0; x < *width; x++) {
	    xx = (int)(x / ratio);
	    /* get source */
	    src = data[yy*icon.Width/2 + xx/2];			 /* get byte */
	    src = (xx & 1) ? (src & 0x0F) : ((src & 0xF0) >> 4); /* get nibble */
	    src = pallette[src];				 /* cvt to pixel */
	    /* get mask */
	    msk = mask[yy*4 + xx/8] & (0x80 >> (xx % 8));	 /* get bit */
	    dst = msk ? bg : src;				 /* set dst */
#ifdef DEBUG_ICON_READ
#endif /* DEBUG_ICON_READ */
	    XPutPixel(image, x, *height -1 - y, dst);
#ifdef DEBUG_ICON_READ
	    printf("%01x", dst);
#endif /* DEBUG_ICON_READ */
	}
#ifdef DEBUG_ICON_READ
	printf("\n");
#endif /* DEBUG_ICON_READ */
    }

    if (!(pixmap = XCreatePixmap (display, DefaultRootWindow (display),
			 *width, *height, 
			 DefaultDepth(display, DefaultScreen(display)))))
	return (0);

    XPutImage (display, pixmap, gc, image, 0, 0,
	   0, 0, *width, *height);

    free (ptr);
    XDestroyImage (image);

    return (pixmap);
}
