/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: RealVal.c,v 1.3 1994/02/18 20:35:56 dta Exp $
 **
 **  $Source: /cvs/lib/xn/RealVal.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#include <stdio.h>
#include <stdlib.h>

#include <Xn/Xn.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>

#ifdef DEBUG
static void     XnPrintReal (soereal)
XnRealAttr    *soereal;
{
    printf ("XnRealAttr %g %g %g %d %d\n",
	    soereal->min,
	    soereal->max,
	    soereal->def,
	    soereal->digits,
	    soereal->accuracy);
}

#endif

static int      VerifyReal (text, soereal)
char           *text;
XnRealAttr    *soereal;
{
    int             val;
    int             len;

    if (!text || !*text) {
	val = soereal->def;
    } else {
	sscanf (text, "%d%n", &val, &len);
	if (!len) {
	    val = soereal->def;
	} else if (val < soereal->min) {
	    val = soereal->min;
	} else if (val > soereal->max) {
	    val = soereal->max;
	}
    }
    return (val);
}

static XtCallbackProc RealModifyVerify (w, soereal, cb)
Widget          w;
XnRealAttr    *soereal;
XmTextVerifyCallbackStruct *cb;
{
    char           *ptr;
    int             len;
    int             i = 0;

#ifdef DEBUG
    printf ("RealModifyVerify %d %d\t", cb->reason, cb->text->length);
    XnPrintReal (soereal);
#endif

    ptr = cb->text->ptr;
    len = cb->text->length;

    if (len) {
	if (*ptr == '-') {
	    ptr++;
	    len--;
	}
	if (len <= soereal->digits) {
	    for (i = 0; i < len; i++) {
		if (!isdigit (ptr[i])) {
		    break;
		}
	    }
	}
    }
    if (i != len)
	cb->doit = False;

    return (0);
}

static XtCallbackProc RealValueChanged (w, soereal, cb)
Widget          w;
XnRealAttr    *soereal;
XmAnyCallbackStruct *cb;
{

#ifdef DEBUG
    printf ("RealValueChanged %d\t", cb->reason);
    XnPrintReal (soereal);
#endif

    return (0);
}

static XtCallbackProc RealLosingFocus (w, soereal, cb)
Widget          w;
XnRealAttr    *soereal;
XmAnyCallbackStruct *cb;
{
    char           *text, str[256];
    int             val;

#ifdef DEBUG
    printf ("RealLosingFocus %d\t", cb->reason);
    XnPrintReal (soereal);
#endif

    if ((text = XmTextGetString (w))) {
	val = VerifyReal (text, soereal);
	sprintf (str, "%d", val);
	if (strcmp (str, text))
	    XmTextSetString (w, str);
	XtFree (text);
    }
    return (0);
}

static XtCallbackProc RealHelp (w, soereal, cb)
Widget          w;
XnRealAttr    *soereal;
XmAnyCallbackStruct *cb;
{

#ifdef DEBUG
    printf ("RealHelp %d\t", cb->reason);
    XnPrintReal (soereal);
#endif

    return (0);
}

static XtCallbackProc RealDestroy (w, soereal, cb)
Widget          w;
XnRealAttr    *soereal;
XmAnyCallbackStruct *cb;
{

#ifdef DEBUG
    printf ("RealDestroy %d\t", cb->reason);
    XnPrintReal (soereal);
#endif

    XtFree ((XtPointer) soereal);
    return (0);
}

/*
 * public functions
 */

Widget          XnCreateVerifiedRealText (parent, min, max, def, digits, accuracy)
Widget          parent;
double          min;
double          max;
double          def;
int             digits;
int             accuracy;
{
    char            value[256];
    XnRealAttr    *soereal;
    Widget          text;

    soereal = (XnRealAttr *) malloc (sizeof (XnRealAttr));

    soereal->min = min;
    soereal->max = max;
    soereal->def = def;
    soereal->digits = digits;
    soereal->accuracy = accuracy;

    sprintf (value, "%.*f", accuracy, def);

    text = XtVaCreateManagedWidget ("RealText",
				    xmTextFieldWidgetClass, parent,
				    XmNmaxLength, digits,
				    XmNcolumns, digits,
				    XmNvalue, value,
				    NULL);

    XnAddCallback (text, XmNmodifyVerifyCallback, RealModifyVerify, soereal);
    XnAddCallback (text, XmNvalueChangedCallback, RealValueChanged, soereal);
    XnAddCallback (text, XmNlosingFocusCallback, RealLosingFocus, soereal);
    XnAddCallback (text, XmNdestroyCallback, RealDestroy, soereal);
    XnAddCallback (text, XmNhelpCallback, RealHelp, soereal);

    return (text);
}
