/*******************************<+>***************************
 **                             DTA
 *************************************************************
 **
 **  $Id: String.c,v 1.6 1994/02/17 03:14:28 dta Exp $
 **
 **  $Source: /cvs/lib/xn/String.c,v $
 **
 **  What @(#):
 **
 **  Author: Dale T. Anderson
 **
 *******************************<+>***************************/
 
#include <Xm/Xm.h>
#include <Xm/Text.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include <Xm/LabelG.h>
#include <Xm/List.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Scale.h>
#include <Xm/ToggleB.h>
#include <Xm/ToggleBG.h>
#include <Xm/MessageB.h>
#include <Xm/DialogS.h>

char *XnWidgetGetStr (Widget w, char *str, const int max)
{
    char *text = NULL;

    *str = 0;

    if (!w)
	fprintf (stderr, "Null widget %s\n", str);
    else if (XmIsText (w))
	text = XmTextGetString (w);
    else if (XmIsTextField (w))
	text = XmTextFieldGetString (w);
    else
	fprintf (stderr, "XnWidgetGetStr - Unknown widget %s\n", str);

    if (text) {
	strcpy (str, text);
	XtFree (text);
    }

    return str;
}

void XnWidgetPutStr (Widget w, const char *str)
{
    if (!w)
	fprintf (stderr, "XnWidgetPutStr - Null widget for %s\n", str);
    else if (XmIsText (w))
	XmTextSetString (w, str);
    else if (XmIsTextField (w))
	XmTextFieldSetString (w, str);
    else if (XmIsDialogShell (w)) {
	XtVaSetValues (w,
	    XmNtitle, str,
	    NULL);
    } else if (XmIsMessageBox (w)) {
	XmString xmstr = XmStringCreateLtoR (str, XmSTRING_DEFAULT_CHARSET);
	XtVaSetValues (w,
	    XmNmessageString, xmstr,
	    NULL);
	XmStringFree (xmstr);
    } else if (XmIsLabel (w)) {
	XtVaSetValues (w,
	    XtVaTypedArg, XmNlabelString, XmRString, str, strlen (str) + 1,
	    NULL);
    } else if (XmIsLabelGadget (w)) {
	XtVaSetValues(w,
	    XtVaTypedArg, XmNlabelString, XmRString, str, strlen (str) + 1,
	    NULL);
    } else if (XmIsPushButton (w) || XmIsPushButtonGadget (w)) {
	XtVaSetValues (w,
	    XtVaTypedArg, XmNlabelString, XmRString, str, strlen (str) + 1,
	    NULL);
    } else if (XmIsToggleButton (w) || XmIsToggleButtonGadget (w)) {
	XtVaSetValues (w,
	    XtVaTypedArg, XmNlabelString, XmRString, str, strlen (str) + 1,
	    NULL);
    } else if (XmIsScale (w)) {
	XtVaSetValues (w,
	    XtVaTypedArg, XmNtitleString, XmRString, str, strlen (str) + 1,
	    NULL);
    } else if (XmIsList (w)) {
	XmString xmstr = XmStringCreate (str, XmSTRING_DEFAULT_CHARSET);
	XmListAddItem (w, xmstr, 0);
	XmStringFree (xmstr);
    } else
	fprintf (stderr, "XnWidgetPutStr - Unknown widget for %s\n", str);
}

