/*******************************<+>***************************
 **                             PRC
 *************************************************************
 **
 **  $Id: Graph.c,v 1.5 1994/02/18 20:35:58 dta Exp $
 **
 **  $Source: /cvs/lib/xn/Widgets/Graph.c,v $
 **
 **  Module Name:
 **
 **  Classification:  Unclassified
 **
 **  Author:          Dale T. Anderson
 **
 *******************************<+>***************************/

#define NOFANSI

#include <memory.h>
#include <string.h>
#include <math.h>

#include <X11/Xlib.h>
#include <X11/Xos.h>
#include <X11/StringDefs.h>

#include <Xm/XmP.h>

#include <Xn/GraphP.h>

#define OffsetOf(field)	XtOffsetOf(XnGraphRec, field)

static XtResource resources[] = {
    {XmNfontList, XmCFontList, XmRFontList, sizeof (XmFontList),
	OffsetOf (graph.font_list),
    XmRImmediate, (XtPointer) NULL},

    {XnNgridLines, XnCGridLines, XmRInt,
	sizeof (int), OffsetOf (graph.grid_lines),
    XmRImmediate, (XtPointer) 1},

    {XnNtotalPoints, XnCTotalPoints, XmRInt,
	sizeof (int), OffsetOf (graph.x1),
    XmRImmediate, (XtPointer) 0},

    {XnNverticalLabels, XnCVerticalLabels, XmRInt,
	sizeof (int), OffsetOf (graph.v_labels),
    XmRImmediate, (XtPointer) 1},

    {XnNhorizontalLabels, XnCHorizontalLabels, XmRInt,
	sizeof (int), OffsetOf (graph.h_labels),
    XmRImmediate, (XtPointer) 1},

    {XnNheaderString, XnCHeaderString, XmRString,
	sizeof (char *), OffsetOf (graph.x1),
    XmRImmediate, (XtPointer) 0},

    {XnNfooterString, XnCFooterString, XmRString,
	sizeof (char *), OffsetOf (graph.x1),
    XmRImmediate, (XtPointer) 0},

    {XnNminimumY, XnCMinimumY, XmRInt,
	sizeof (int), OffsetOf (graph.x1),
    XmRImmediate, (XtPointer) 0},

    {XnNmaximumY, XnCMaximumY, XmRInt,
	sizeof (int), OffsetOf (graph.x1),
    XmRImmediate, (XtPointer) 0},
};

#undef OffsetOf

/*
 * Declaration of methods
 */

static void     Initialize ();
static Boolean  SetValues ();
static void     Resize ();
static void     redisplay_graph ();

/*
 * Private functions
 */

XnGraphClassRec xnGraphClassRec = {
    {
	/* core_class fields */
	 /* superclass			 */ (WidgetClass) & xmPrimitiveWidgetClass,
	 /* class_name			 */ "XnGraph",
	 /* widget_size			 */ sizeof (XnGraphRec),
	 /* class_initialize		 */ NULL,
	 /* class_part_initialize	 */ NULL,
	 /* class_inited		 */ False,
	 /* initialize			 */ Initialize,
	 /* initialize_hook		 */ NULL,
	 /* realize			 */ XtInheritRealize,
	 /* actions			 */ NULL,
	 /* num_actions			 */ 0,
	 /* resources			 */ resources,
	 /* num_resources		 */ XtNumber (resources),
	 /* xrm_class			 */ NULLQUARK,
	 /* compress_motion		 */ True,
	 /* compress_exposure		 */ True,
	 /* compress_enterleave		 */ True,
	 /* visible_interest		 */ True,
	 /* destroy			 */ NULL,
	 /* resize			 */ Resize,
	 /* expose			 */ redisplay_graph,
	 /* set_values			 */ SetValues,
	 /* set_values_hook		 */ NULL,
	 /* set_values_almost		 */ XtInheritSetValuesAlmost,
	 /* get_values_hook		 */ NULL,
	 /* accept_focus		 */ NULL,
	 /* version			 */ XtVersion,
	 /* callback_private		 */ NULL,
	 /* tm_table			 */ XtInheritTranslations,
	 /* query_geometry		 */ NULL,
	 /* display_accelerator		 */ NULL,
	 /* extension			 */ NULL
    },
    {
	 /* drawing area class part    	 */ NULL
    },
    {
	/* matrix_class fields */
	 /* extension			 */ NULL,
    }
};

WidgetClass     xnGraphWidgetClass = (WidgetClass) & xnGraphClassRec;

#define stipple_width 16
#define stipple_height 16
static char     stipple_bits[] = {
    0x11, 0x11, 0x0a, 0x0a, 0x00, 0x00, 0x0a, 0x0a, 0x11, 0x11, 0xa0, 0xa0,
    0x00, 0x00, 0xa0, 0xa0, 0x11, 0x11, 0x0a, 0x0a, 0x00, 0x00, 0x0a, 0x0a,
0x11, 0x11, 0xa0, 0x20, 0x00, 0x40, 0xa0, 0x80};

static Pixmap   stipple = 0;

static void     CreatePixmap (gw)
XnGraphWidget   gw;
{
    Display        *display = XtDisplay (gw);
    int             screen = DefaultScreen (display);

    gw->graph.pixmap = XCreatePixmap (display,
				      RootWindow (display, screen),
				      gw->core.width, gw->core.height,
				      DefaultDepth (display, screen));
}

static void     CreateStipple (gw)
XnGraphWidget   gw;
{
    Display        *display = XtDisplay (gw);
    int             screen = DefaultScreen (display);

    if (!stipple)
	stipple = XCreatePixmapFromBitmapData (display, gw->graph.pixmap,
				stipple_bits, stipple_width, stipple_height,
					       BlackPixel (display, screen),
					       WhitePixel (display, screen),
					       1);
}

static void     CreateGC (gw)
XnGraphWidget   gw;
{
    gw->graph.gc = XCreateGC (XtDisplay (gw), gw->graph.pixmap, 0, NULL);
}

static void     CreateColor (gw)
XnGraphWidget   gw;
{
    Display        *display = XtDisplay (gw);
    int             screen = DefaultScreen (display);
    XColor          def;
    XColor          rgb;

    XAllocNamedColor (XtDisplay (gw),
		      DefaultColormap (display, screen),
		      "gray", &def, &rgb);
    gw->graph.bg = BlackPixel (display, screen);
    gw->graph.fg = def.pixel;
}

void            GraphRemove (gw, id)
XnGraphWidget   gw;
int             id;
{

#ifdef DEBUG
    printf ("GraphRemove id %d, count = %d\n", id, gw->graph.count);
#endif

    if (id < gw->graph.count) {
	if (gw->graph.f[id].data)
	    XtFree (gw->graph.f[id].data);
	if (gw->graph.f[id].format)
	    XtFree (gw->graph.f[id].format);
	if (gw->graph.f[id].header)
	    XtFree (gw->graph.f[id].header);
	if (gw->graph.count == 1) {
	    XtFree (gw->graph.f);
	    gw->graph.f = NULL;
	}
	gw->graph.count--;
    }
}

static void     graph_lines (gw)
XnGraphWidget   gw;
{
    Display        *display = XtDisplay (gw);
    int             screen = DefaultScreen (display);
    int             x, y;

    XSetForeground (display, gw->graph.gc, WhitePixel (display, screen));
    XDrawRectangle (display, gw->graph.pixmap, gw->graph.gc,
		    gw->graph.x1 - 1, gw->graph.y1 - 1,
	  gw->graph.x2 - gw->graph.x1 + 2, gw->graph.y2 - gw->graph.y1 + 2);

    XSetForeground (display, gw->graph.gc, gw->graph.fg);
    XSetLineAttributes (display, gw->graph.gc, 1,
			LineOnOffDash, JoinRound, FillSolid);

    for (y = gw->graph.y2 - gw->graph.yspace; y > gw->graph.y1; y -= gw->graph.yspace)
	XDrawLine (display, gw->graph.pixmap, gw->graph.gc,
		   gw->graph.x1, y, gw->graph.x2, y);

    for (x = gw->graph.x1 + gw->graph.xspace; x < gw->graph.x2; x += gw->graph.xspace)
	XDrawLine (display, gw->graph.pixmap, gw->graph.gc,
		   x, gw->graph.y2, x, gw->graph.y1);

    XSetLineAttributes (display, gw->graph.gc, 1,
			LineSolid, JoinRound, FillSolid);
}

static void     graph_borders (gw, f)
XnGraphWidget   gw;
XnFunction     *f;
{
    char            buffer[256];
    XFontStruct    *font;
    int             x;

    font = XQueryFont (XtDisplay (gw), XGContextFromGC (gw->graph.gc));

    if (f->header)
	gw->graph.y1 = 10 + font->descent + font->ascent;
    else
	gw->graph.y1 = 10;

    if (f->footer)
	gw->graph.y2 = gw->core.height - gw->graph.y1;
    else
	gw->graph.y2 = gw->core.height - 10;

    gw->graph.x2 = gw->core.width - 10;
    gw->graph.x1 = 0;

    sprintf (buffer, f->format, f->factor * f->min);
    x = XTextWidth (font, buffer, strlen (buffer));
    if (x + 10 > gw->graph.x1)
	gw->graph.x1 = x + 10;

    sprintf (buffer, f->format, f->factor * f->max);
    x = XTextWidth (font, buffer, strlen (buffer));
    if (x + 10 > gw->graph.x1)
	gw->graph.x1 = x + 10;
}

static void     graph_labels (gw, f)
XnGraphWidget   gw;
XnFunction     *f;
{
    Display        *display = XtDisplay (gw);
    int             screen = DefaultScreen (display);
    XFontStruct    *font;
    char            buffer[256];
    static int      vals[] = {2, 5, 10, 15, 20, 25, 50, 100};
    int             space;
    int             i = 0;
    int             x, y;
    float           pratio;
    float           gratio;
    float           gcount;

    pratio = (float) (gw->graph.y2 - gw->graph.y1 + 1) / (float) (f->max - f->min + 1);

    do {
	if (i < sizeof (vals) / sizeof (int))
	    space = vals[i++];
	else
	    space += 100;
    } while (space * pratio < 20);

#ifdef DEBUG
    printf ("Space is at %d\n", space);
#endif

    font = XQueryFont (display, XGContextFromGC (gw->graph.gc));

    gratio = (float) (space) / (float) (f->max - f->min);
    gcount = (f->max - f->min + 1) / space;

    pratio = (float) (gw->graph.y2 - gw->graph.y1 + 1) / (float) (gcount);

    if (f->header) {
	XSetForeground (display, gw->graph.gc, WhitePixel (display, screen));

	x = (gw->core.width - XTextWidth (font, f->header, strlen (f->header))) / 2;
	XDrawString (display, gw->graph.pixmap, gw->graph.gc,
		     x, gw->graph.y1 / 2 + font->ascent / 2 - 1,
		     f->header, strlen (f->header));
    }
    for (i = 0; i <= gcount; i++) {
	y = (gw->graph.y2 + 1) - i * pratio;

	XSetForeground (display, gw->graph.gc, WhitePixel (display, screen));

	XDrawLine (display, gw->graph.pixmap, gw->graph.gc,
		   gw->graph.x1 - 4, y, gw->graph.x1, y);

	XSetForeground (display, gw->graph.gc, f->fg);

	sprintf (buffer, "%g", f->factor * (f->min + i * space));
	x = gw->graph.x1 - XTextWidth (font, buffer, strlen (buffer)) - 6;
	XDrawString (display, gw->graph.pixmap, gw->graph.gc,
		     x, y + font->ascent / 2 - 1, buffer, strlen (buffer));
    }
}

static void     graph_plot (gw, f)
XnGraphWidget   gw;
XnFunction     *f;
{
    int             i;
    XPoint         *points;

    if (f->max == f->min)
	return;

    f->yratio = (float) (gw->graph.y2 - gw->graph.y1 + 1) / (float) (f->max - f->min);
    f->xratio = (float) (gw->graph.x2 - gw->graph.x1 + 1)
	/ (float) (f->data[f->count - 1].x - f->data[0].x);

    points = (XPoint *) XtMalloc (sizeof (XPoint) * (f->count + 2));

    for (i = 0; i < f->count; i++) {
	points[i].x = (int) ((f->data[i].x - f->data[0].x) * f->xratio) + gw->graph.x1;
	points[i].y = gw->graph.y2 - (int) ((f->data[i].y - f->min) * f->yratio);
    }

    XSetForeground (XtDisplay (gw), gw->graph.gc, f->fg);

    if (f->filled) {
	XSetFillRule (XtDisplay (gw), gw->graph.gc, EvenOddRule);
	XSetFillStyle (XtDisplay (gw), gw->graph.gc, FillStippled);
	XSetStipple (XtDisplay (gw), gw->graph.gc, stipple);

	points[i].x = gw->graph.x2;
	points[i].y = gw->graph.y2;
	i++;
	points[i].x = gw->graph.x1;
	points[i].y = gw->graph.y2;
	i++;

	XFillPolygon (XtDisplay (gw), gw->graph.pixmap, gw->graph.gc,
		      points, i, Complex, CoordModeOrigin);

	XSetFillStyle (XtDisplay (gw), gw->graph.gc, FillSolid);
    }
    XDrawLines (XtDisplay (gw), gw->graph.pixmap, gw->graph.gc, points, i, CoordModeOrigin);

    XtFree (points);
}

void            GraphSetMinMax (gw, id, min, max)
XnGraphWidget   gw;
int             id;
int             min;
int             max;
{

#ifdef DEBUG
    printf ("GraphSetMinMax min %d, max %d\n", min, max);
#endif

    if (id < gw->graph.count) {
	gw->graph.f[id].min = min;
	gw->graph.f[id].max = max;
    }
}

void            GraphSetHeader (gw, id, header)
XnGraphWidget   gw;
int             id;
char           *header;
{
    if (id < gw->graph.count)
	gw->graph.f[id].header = strdup (header);
}

void            DeleteGraph (gw)
XnGraphWidget   gw;
{
    int             i;

    if (gw) {
	for (i = 0; i < gw->graph.count; i++)
	    GraphRemove (gw, i);
	XtFree ((char *) (gw));
    }
}

void            GraphAddFunction (gw, data, count, color, format, factor)
XnGraphWidget   gw;
XPoint         *data;
int             count;
char           *color;
char           *format;
double          factor;
{
    XnFunction     *f;
    XColor          def;
    XColor          rgb;
    int             i;

    if (!gw->graph.f) {
	gw->graph.count = 1;
	gw->graph.f = (XnFunction *) XtMalloc (sizeof (XnFunction));
    } else {
	gw->graph.count++;
	gw->graph.f = (XnFunction *) XtRealloc ((char *) gw->graph.f, sizeof (XnFunction) * gw->graph.count);
    }

    f = gw->graph.f + gw->graph.count - 1;

    (void) memset ((char *) f, 0, sizeof (XnFunction));

    f->count = 0;
    f->factor = factor;
    f->min = f->max = 0;
    f->xratio = 1.0;
    f->yratio = 1.0;
    f->format = strdup (format);
    f->filled = 1;

    if (count && (f->data = (XPoint *) XtMalloc (count * sizeof (XPoint)))) {
	memcpy ((char *) f->data, data, sizeof (XPoint) * count);

	f->count = count;

	f->min = f->data[0].y;
	f->max = f->data[0].y;

	for (i = 0; i < count; i++) {
	    if (f->data[i].y < f->min)
		f->min = f->data[i].y;
	    if (f->data[i].y > f->max)
		f->max = f->data[i].y;
	}
    }
    if (XAllocNamedColor (XtDisplay (gw),
	   DefaultColormap (XtDisplay (gw), DefaultScreen (XtDisplay (gw))),
			  color, &def, &rgb) == BadName)
	f->fg = gw->graph.fg;
    else
	f->fg = def.pixel;

#ifdef DEBUG
    printf ("GraphAdd min %d, max %d, count %d\n", f->min, f->max, count);
#endif
}

static void     Initialize (request, new, args, num_args)
XnGraphWidget   request, new;
ArgList         args;
Cardinal       *num_args;
{

#ifdef DEBUG
    printf ("Initialize\n");
#endif

    if (new->core.width < 300)
	new->core.width = 300;
    if (new->core.height < 150)
	new->core.height = 150;

    new->graph.xspace = 50;
    new->graph.yspace = 25;

    new->graph.x1 = 10;
    new->graph.y1 = 10;
    new->graph.x2 = new->core.width - 10;
    new->graph.y2 = new->core.height - 10;

    new->graph.selected = 0;
    new->graph.count = 0;

    CreatePixmap (new);
    CreateStipple (new);
    CreateGC (new);
    CreateColor (new);
}

#define NE(field)       (current->field != new->field)
#define EQ(field)       (current->field == new->field)

static Boolean  SetValues (current, request, new, args, num_args)
XnGraphWidget   current, request, new;
ArgList         args;
Cardinal       *num_args;
{

#ifdef DEBUG
    printf ("SetValues\n");
#endif

#if 0
    if (NE (graph.count)) {
	printf ("count %d -> %d\n", current->graph.count, new->graph.count);
    }
#endif

    return False;
}

#undef EQ
#undef NE

static void     Resize (gw)
XnGraphWidget   gw;
{

#ifdef DEBUG
    printf ("Resize %d x %d\n", gw->core.width, gw->core.height);
#endif

    if (gw->graph.pixmap)
	XFreePixmap (XtDisplay (gw), gw->graph.pixmap);

    CreatePixmap (gw);
}

static void            redisplay_graph (gw, event, region)
XnGraphWidget   gw;
XEvent         *event;
Region          region;
{

#ifdef DEBUG
    printf ("redisplay_graph\n");
#endif

    XSetWindowBackgroundPixmap (XtDisplay (gw), XtWindow (gw), gw->graph.pixmap);
    XSetForeground (XtDisplay (gw), gw->graph.gc, gw->graph.bg);
    XFillRectangle (XtDisplay (gw), gw->graph.pixmap, gw->graph.gc, 0, 0, gw->core.width, gw->core.height);

    gw->graph.x2 = gw->core.width - 10;
    gw->graph.y2 = gw->core.height - 10;

    if (gw->graph.count) {
	graph_borders (gw, gw->graph.f[gw->graph.selected]);
	graph_labels (gw, gw->graph.f[gw->graph.selected]);
	gw->graph.selected = ++gw->graph.selected % gw->graph.count;
    }
    graph_lines (gw);

    XClearWindow (XtDisplay (gw), XtWindow (gw));

    if (gw->graph.count) {
	int             i;

	for (i = 0; i < gw->graph.count; i++) {
	    graph_plot (gw, gw->graph.f + i);
	    XClearWindow (XtDisplay (gw), XtWindow (gw));
	}
    }
}
