/*******************************<+>***************************
 **                             DTA
 *************************************************************
 **
 **  $Id: Print.C,v 1.4 1994/02/18 22:25:55 dta Exp $
 **
 **  $Source: /cvs/progs/pexdb/Print.C,v $
 **
 **  What @(#):
 **
 **  Author: Dale T. Anderson
 **
 *******************************<+>***************************/
 
#include <stdio.h>

#include <Xn/Xn.h>

#include "Pexdb.h"

//
// Intrinsic types
//

void Print (Widget w, const char *value, const char *str)
{
    AddToList (w, "%-8s %s", str, value);
}

void Print (Widget w, const short value, const char *str)
{
    AddToList (w, "%-8s %d", str, value);
}

void Print (Widget w, const unsigned short value, const char *str)
{
    AddToList (w, "%-8s %d", str, value);
}

void Print (Widget w, const int value, const char *str)
{
    AddToList (w, "%-8s %10d", str, value);
}

void Print (Widget w, const unsigned int value, const char *str)
{
    AddToList (w, "%-8s %10d", str, value);
}

void Print (Widget w, const long value, const char *str)
{
    AddToList (w, "%-8s %10d", str, value);
}

void Print (Widget w, const unsigned long value, const char *str)
{
    AddToList (w, "%-8s %10ld", str, value);
}

void Print (Widget w, const float value, const char *str)
{
    AddToList (w, "%-8s %10.3f", str, value);
}

void Print (Widget w, const double value, const char *str)
{
    AddToList (w, "%-8s %10.3f", str, value);
}

//
// Vectors, Coords
//

void Print (Widget w, const PEXVector &v, const char *str)
{
    AddToList (w, "%-8s %10.3f %10.3f %10.3f", str, v.x, v.y, v.z);
}

void Print (Widget w, const PEXVector2D &v, const char *str)
{
    AddToList (w, "%-8s %10.3f %10.3f", str, v.x, v.y);
}

void Print (Widget w, const PEXCoord &c, const char *str)
{
    AddToList (w, "%-8s %10.3f %10.3f %10.3f", str, c.x, c.y, c.z);
}

void Print (Widget w, const PEXCoord2D &c, const char *str)
{
    AddToList (w, "%-8s %10.3f %10.3f", str, c.x, c.y);
}

void Print (Widget w, const PEXCoord4D &c, const char *str)
{
    AddToList (w, "%-8s %10.3f %10.3f %10.3f %10.3f", str, c.x, c.y, c.z, c.w);
}

//
// Vertexes
//

void Print (Widget w, const PEXVertexIndexed &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.index, "index");
}

void Print (Widget w, const PEXVertexRGB &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb, "rgb");
}

void Print (Widget w, const PEXVertexHSV &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hsv, "hsv");
}

void Print (Widget w, const PEXVertexHLS &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hls, "hls");
}

void Print (Widget w, const PEXVertexCIE &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.cie, "cie");
}

void Print (Widget w, const PEXVertexRGB8 &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb8, "rgb8");
}

void Print (Widget w, const PEXVertexRGB16 &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb16, "rgb16");
}

void Print (Widget w, const PEXVertexNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexIndexedNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.index, "index");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexRGBNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb, "rgb");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexHSVNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hsv, "hsv");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexHLSNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hls, "hls");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexCIENormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.cie, "cie");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexRGB8Normal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb8, "rgb8");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexRGB16Normal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb16, "rgb16");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXVertexIndexedEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.index, "index");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexRGBEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb, "rgb");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexHSVEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hsv, "hsv");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexHLSEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hls, "hls");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexCIEEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.cie, "cie");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexRGB8Edge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb8, "rgb8");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexRGB16Edge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb16, "rgb16");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexNormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexIndexedNormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.index, "index");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexRGBNormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb, "rgb");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexHSVNormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hsv, "hsv");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexHLSNormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.hls, "hls");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexCIENormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.cie, "cie");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexRGB8NormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb8, "rgb8");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

void Print (Widget w, const PEXVertexRGB16NormalEdge &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.point, "point");
    Print (w, v.rgb16, "rgb16");
    Print (w, v.normal, "normal");
    Print (w, v.edge, "edge");
}

//
// Facets
//

void Print (Widget w, const PEXColorIndexedNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.index, "index");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXColorRGBNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.rgb, "rgb");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXColorCIENormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.cie, "cie");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXColorHSVNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.hsv, "hsv");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXColorHLSNormal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.hls, "hls");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXColorRGB8Normal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.rgb8, "rgb8");
    Print (w, v.normal, "normal");
}

void Print (Widget w, const PEXColorRGB16Normal &v, const char *str)
{
    AddToList (w, str);
    Print (w, v.rgb16, "rgb16");
    Print (w, v.normal, "normal");
}

//
// Colors
//

void Print (Widget w, const PEXColorIndexed &color, const char *str)
{
    AddToList (w, str);
    Print (w, color.index, "index");
    Print (w, color.reserved, "reserved");
}

void Print (Widget w, const PEXColorRGB &color, const char *str)
{
    AddToList (w, "%-8s RGB %10.3f %10.3f %10.3f", str,
	color.red, color.green, color.blue);
}

void Print (Widget w, const PEXColorHSV &color, const char *str)
{
    AddToList (w, "%-8s HSV %10.3f %10.3f %10.3f", str,
	color.hue, color.saturation, color.value);
}

void Print (Widget w, const PEXColorHLS &color, const char *str)
{
    AddToList (w, "%-8s HLS %10.3f %10.3f %10.3f", str,
	color.hue, color.lightness, color.saturation);
}

void Print (Widget w, const PEXColorCIE &color, const char *str)
{
    AddToList (w, "%-8s CIE %10.3f %10.3f %10.3f", str,
	color.x, color.y, color.z);
}

void Print (Widget w, const PEXColorRGB8 &color, const char *str)
{
    AddToList (w, "%-8s RGB8 %10d %10d %10d", str,
	color.red, color.green, color.blue);
}

void Print (Widget w, const PEXColorRGB16 &color, const char *str)
{
    AddToList (w, "%-8s RGB16 %10d %10d %10d", str,
	color.red, color.green, color.blue);
}

//
// Lists
//

void Print (Widget w, const PEXListOfCoord &coords, const char *str)
{
    AddToList (w, str);
    Print (w, coords.count, "count");
    char buf [32];
    for (int i = 0; i < coords.count; i++) {
	sprintf (buf, "%3d", i);
	Print (w, coords.points [i], buf);
    }
}

void Print (Widget w, const PEXListOfCoord2D &coords, const char *str)
{
    AddToList (w, str);
    Print (w, coords.count, "count");
    char buf [32];
    for (int i = 0; i < coords.count; i++) {
	sprintf (buf, "%3d", i);
	Print (w, coords.points [i], buf);
    }
}

void Print (Widget w, const PEXListOfTrimCurve &list, const char *str)
{
    AddToList (w, str);
    Print (w, list.count, "count");
    char buf [32];
    for (int i = 0; i < list.count; i++) {
	sprintf (buf, "%3d", i);
	Print (w, list.curves [i], buf);
    }
}

void Print (Widget w, const int type, const PEXListOfVertex &list, const char *str)
{
    AddToList (w, str);
    Print (w, list.count, "count");
    Print (w, type, list.vertices, list.count, "vertices");
}

void Print (Widget w, const PEXListOfUShort &list, const char *str)
{
    AddToList (w, str);
    Print (w, list.count, "count");
    char buf [32];
    for (int i = 0; i < list.count; i++) {
	sprintf (buf, "%3d", i);
	Print (w, list.shorts [i], buf);
    }
}

//
// Miscellaneous structures
//

void Print (Widget w, const PEXHalfSpace &space, const char *str)
{
    AddToList (w, str);
    Print (w, space.point, "coord");
    Print (w, space.vector, "vector");
}

void Print (Widget w, const PEXHalfSpace2D &space, const char *str)
{
    AddToList (w, str);
    Print (w, space.point, "coord");
    Print (w, space.vector, "vector");
}

void Print (Widget w, const PEXConnectivityData &data, const char *str)
{
    AddToList (w, str);
    Print (w, data.count, "count");
    for (int i = 0; i < data.count; i++)
	Print (w, data.lists [i], "lists");
}

void Print (Widget w, const PEXEncodedTextData *, const char *str)
{
    AddToList (w, str);
}

void Print (Widget w, const int type, const PEXFacetData &, const char *str)
{
    AddToList (w, str);
}

void Print (Widget w, const PEXMatrix &matrix, const char *str)
{
    AddToList (w, str);
    for (int i = 0; i < 4; i++)
	AddToList (w, "%10.3f %10.3f %10.3f %10.3f",
	    matrix [0][i], matrix [1][i], matrix [2][i], matrix [3][i]);
}

void Print (Widget w, const PEXMatrix3x3 &matrix, const char *str)
{
    AddToList (w, str);
    for (int i = 0; i < 3; i++)
	AddToList (w, "%10.3f %10.3f %10.3f",
	    matrix [0][i], matrix [1][i], matrix [2][i]);
}

void Print (Widget w, const PEXPSCData &, const char *str)
{
    AddToList (w, str);
}

void Print (Widget w, const PEXReflectionAttributes &attr, const char *str)
{
    AddToList (w, str);
    Print (w, attr.ambient, "ambient");
    Print (w, attr.diffuse, "diffuse");
    Print (w, attr.specular, "specular");
    Print (w, attr.specular_conc, "specular_conc");
    Print (w, attr.transmission, "transmission");
}

void Print (Widget w, const PEXTrimCurve &curve, const char *str)
{
    AddToList (w, str);
    Print (w, curve.visibility, "visibility");
    Print (w, curve.reserved, "reserved");
    Print (w, curve.order, "order");
    Print (w, curve.rationality, "rationality");
    Print (w, curve.approx_method, "approx_method");
    Print (w, curve.tolerance, "tolerance");
    Print (w, curve.tmin, "tmin");
    Print (w, curve.tmax, "tmax");
    Print (w, curve.count, "count");
}

//
// Unions
//

void Print (Widget w, const char *str, const int type, const PEXColor &color)
{
    switch (type) {
	case PEXColorTypeIndexed:
	    Print (w, color.indexed, str);
	    break;
	case PEXColorTypeRGB:
	    Print (w, color.rgb, str);
	    break;
	case PEXColorTypeCIE:
	    Print (w, color.cie, str);
	    break;
	case PEXColorTypeHSV:
	    Print (w, color.hsv, str);
	    break;
	case PEXColorTypeHLS:
	    Print (w, color.hls, str);
	    break;
	case PEXColorTypeRGB8:
	    Print (w, color.rgb8, str);
	    break;
	case PEXColorTypeRGB16:
	    Print (w, color.rgb16, str);
	    break;
    }
}

//
// Arrays
//

void Print (Widget w, const int type, PEXArrayOfColor &array, const int count, const char *str)
{
    AddToList (w, str);
    char buf [256];
    for (int i = 0; i < count; i++) {
	sprintf (buf, "%3d", i);
	switch (type) {
	    case PEXColorTypeIndexed:
		Print (w, array.indexed[i], str);
		break;
	    case PEXColorTypeRGB:
		Print (w, array.rgb[i], str);
		break;
	    case PEXColorTypeCIE:
		Print (w, array.cie[i], str);
		break;
	    case PEXColorTypeHSV:
		Print (w, array.hsv[i], str);
		break;
	    case PEXColorTypeHLS:
		Print (w, array.hls[i], str);
		break;
	    case PEXColorTypeRGB8:
		Print (w, array.rgb8[i], str);
		break;
	    case PEXColorTypeRGB16:
		Print (w, array.rgb16[i], str);
		break;
	}
    }
}

void Print (Widget w, const int type, const PEXArrayOfCoord &array, const int count, const char *str)
{
    AddToList (w, str);
    char buf [256];
    for (int i = 0; i < count; i++) {
	sprintf (buf, "%3d", i);
	switch (type) {
	    case 0:
		Print (w, array.point_2d [i], buf);
		break;
	    case 1:
		Print (w, array.point [i], buf);
		break;
	    case 2:
		Print (w, array.point_4d [i], buf);
		break;
	}
    }
}

void Print (Widget w, const int type, const PEXArrayOfVertex &array, const int count, const char *str)
{
    AddToList (w, str);
    char buf [256];
    for (int i = 0; i < count; i++) {
	sprintf (buf, "%3d", i);
	switch (type) {
	    case 0: // Coord
		//Print (w, array.no_dta[i], buf);
		break;
	    case 1: // VertexIndexed
		Print (w, array.index[i], buf);
		break;
	    case 2: // VertexRGB
		Print (w, array.rgb[i], buf);
		break;
	    case 3: // VertexHSV
		Print (w, array.hsv[i], buf);
		break;
	    case 4: // VertexHLS
		Print (w, array.hls[i], buf);
		break;
	    case 5: // VertexCIE
		Print (w, array.cie[i], buf);
		break;
	    case 6: // VertexRGB8
		Print (w, array.rgb8[i], buf);
		break;
	    case 7: // VertexRGB16
		Print (w, array.rgb16[i], buf);
		break;
	    case 8: // VertexNormal
		Print (w, array.normal[i], buf);
		break;
	    case 9: // VertexEdge
		Print (w, array.edge[i], buf);
		break;
	    case 10: // VertexIndexedNormal
		Print (w, array.index_normal[i], buf);
		break;
	    case 11: // VertexRGBNormal
		Print (w, array.rgb_normal[i], buf);
		break;
	    case 12: // VertexHSVNormal
		Print (w, array.hsv_normal[i], buf);
		break;
	    case 13: // VertexHLSNormal
		Print (w, array.hls_normal[i], buf);
		break;
	    case 14: // VertexCIENormal
		Print (w, array.cie_normal[i], buf);
		break;
	    case 15: // VertexRGB8Normal
		Print (w, array.rgb8_normal[i], buf);
		break;
	    case 16: // VertexRGB16Normal
		Print (w, array.rgb16_normal[i], buf);
		break;
	    case 17: // VertexIndexedEdge
		Print (w, array.index_edge[i], buf);
		break;
	    case 18: // VertexRGBEdge
		Print (w, array.rgb_edge[i], buf);
		break;
	    case 19: // VertexHSVEdge
		Print (w, array.hsv_edge[i], buf);
		break;
	    case 20: // VertexHLSEdge
		Print (w, array.hls_edge[i], buf);
		break;
	    case 21: // VertexCIEEdge
		Print (w, array.cie_edge[i], buf);
		break;
	    case 22: // VertexRGB8Edge
		Print (w, array.rgb8_edge[i], buf);
		break;
	    case 23: // VertexRGB16Edge
		Print (w, array.rgb16_edge[i], buf);
		break;
	    case 24: // VertexNormalEdge
		Print (w, array.normal_edge[i], buf);
		break;
	    case 25: // VertexIndexedNormalEdge
		Print (w, array.index_normal_edge[i], buf);
		break;
	    case 26: // VertexRGBNormalEdge
		Print (w, array.rgb_normal_edge[i], buf);
		break;
	    case 27: // VertexHSVNormalEdge
		Print (w, array.hsv_normal_edge[i], buf);
		break;
	    case 28: // VertexHLSNormalEdge
		Print (w, array.hls_normal_edge[i], buf);
		break;
	    case 29: // VertexCIENormalEdge
		Print (w, array.cie_normal_edge[i], buf);
		break;
	    case 30: // VertexRGB8NormalEdge
		Print (w, array.rgb8_normal_edge[i], buf);
		break;
	    case 31: // VertexRGB16NormalEdge
		Print (w, array.rgb16_normal_edge[i], buf);
		break;
	}
    }
}

void Print (Widget w, const int type, const PEXArrayOfFacetData &array, const int count, const char *str)
{
    AddToList (w, str);
    Print (w, count, "count");
    char buf [256];
    for (int i = 0; i < count; i++) {
	sprintf (buf, "%3d", i);
	switch (type) {
	    case 0: // ColorIndexed
		Print (w, array.index[i], buf);
		break;
	    case 1: // ColorRGB
		Print (w, array.rgb[i], buf);
		break;
	    case 2: // ColorHSV
		Print (w, array.hsv[i], buf);
		break;
	    case 3: // ColorHLS
		Print (w, array.hls[i], buf);
		break;
	    case 4: // ColorCIE
		Print (w, array.cie[i], buf);
		break;
	    case 5: // ColorRGB8
		Print (w, array.rgb8[i], buf);
		break;
	    case 6: // ColorRGB16
		Print (w, array.rgb16[i], buf);
		break;
	    case 7: // Vector
		Print (w, array.normal[i], buf);
		break;
	    case 8: // ColorIndexedNormal
		Print (w, array.index_normal[i], buf);
		break;
	    case 9: // ColorRGBNormal
		Print (w, array.rgb_normal[i], buf);
		break;
	    case 10: // ColorCIENormal
		Print (w, array.cie_normal[i], buf);
		break;
	    case 11: // ColorHSVNormal
		Print (w, array.hsv_normal[i], buf);
		break;
	    case 12: // ColorHLSNormal
		Print (w, array.hls_normal[i], buf);
		break;
	    case 13: // ColorRGB8Normal
		Print (w, array.rgb8_normal[i], buf);
		break;
	    case 14: // ColorRGB16Normal
		Print (w, array.rgb16_normal[i], buf);
		break;
	}
    }
}

